<?php
/**
 * API '/payments' resource.
 *
 * @package WooAsaas
 */

namespace WC_Asaas\Api\Resources;

use WC_Asaas\Api\Response\Response;
use WC_Asaas\Api\Client\Client;
use WC_Asaas\Api\Client\Collection_Client;

/**
 * API '/payments' resource.
 */
class Payments extends Resource {

	/**
	 * Resource path.
	 *
	 * @var string
	 */
	const PATH = '/payments/';

	/**
	 * Find a payment by id.
	 *
	 * @param  int $id Payment id.
	 * @return Response The HTTP response.
	 */
	public function find( $id ) {
		$client = new Client( $this->gateway );
		return $client->get( self::PATH . $id );
	}

	/**
	 * Get all payments.
	 *
	 * @return Response The HTTP response.
	 */
	public function all() {
		$client = new Collection_Client( $this->gateway );
		return $client->get( self::PATH );
	}

	/**
	 * Update a payment by id.
	 *
	 * @param  int   $id Payment id.
	 * @param  array $data Request body.
	 * @return Response The HTTP response.
	 */
	public function update( $id, $data ) {
		$client = new Client( $this->gateway );
		return $client->post( self::PATH, $data, array( $this, 'filter_data_log' ) );
	}

	/**
	 * Create a newly payment.
	 *
	 * @param  array $data Request body.
	 * @return Response The HTTP response.
	 */
	public function create( $data ) {
		$client = new Client( $this->gateway );
		return $client->post( self::PATH, $data, array( $this, 'filter_data_log' ) );
	}

	/**
	 * Delete a payment by id.
	 *
	 * @param  int $id Payment id.
	 * @return Response The HTTP response.
	 */
	public function delete( $id ) {
		$client = new Client( $this->gateway );
		return $client->delete( self::PATH . $id );
	}

	/**
	 * Refund a payment by id.
	 *
	 * @param  int $id Payment id.
	 * @return Response The HTTP response.
	 */
	public function refund( $id ) {
		$client = new Client( $this->gateway );
		return $client->delete( self::PATH . $id . 'refund' );
	}

	/**
	 * Get the minimum bank slip due date
	 *
	 * @return Response The HTTP response.
	 */
	public function minimum_bank_slip_due_date() {
		$client = new Client( $this->gateway );
		return $client->get( self::PATH . 'minimumBankSlipDueDate' );
	}

	/**
	 * Remove sensitive card and holder data to not be stored in log
	 *
	 * @param string|\stdClass $data The data to be stored.
	 * @return string|false The data encoded on string.
	 */
	public function filter_data_log( $data ) {
		if ( is_string( $data ) ) {
			$data = json_decode( $data, true );
		}

		if ( isset( $data['creditCard'] ) ) {
			unset( $data['creditCard'] );
		}

		if ( ! empty( $data['creditCardHolderInfo'] ) ) {
			unset( $data['creditCardHolderInfo'] );
		}

		return wp_json_encode( $data );
	}
}
