<?php
/**
 * Credit Card class
 *
 * @package WooAsaas
 */

namespace WC_Asaas\Gateway;

use WC_Asaas\Meta_Data\Order;
use WC_Asaas\Meta_Data\Customer;
use WC_Asaas\Admin\Settings\Credit_Card as Credit_Card_Settings;
use WC_Asaas\Billing_Type\Credit_Card as Credit_Card_Type;
use WC_Asaas\Api\Response\Error_Response;
use WC_Asaas\Checkout\Form_Field\Card;
use WC_Asaas\WC_Asaas;
use WC_Asaas\Helper\Credit_Card_Helper;
use WC_Asaas\Helper\Validation_Helper;
use WC_Asaas\Checkout\Form_Field\One_Click_Options;
use WC_Asaas\Helper\Installments_Helper;

/**
 * Asaas credit card gateway
 */
class Credit_Card extends Gateway {

	/**
	 * Cache to the card brand of the submitted card
	 *
	 * @var array
	 */
	private $checkout_card_brand;

	/**
	 * Init the gateway
	 */
	public function __construct() {
		$this->id           = 'asaas-credit-card';
		$this->has_fields   = true;
		$this->method_title = __( 'Asaas Credit Card', 'woo-asaas' );
		/* translators: %s: Asaas website URL  */
		$this->method_description = sprintf( __( 'Use <a href="%s">Asaas</a> to allow your customer buy your products in installments using credit card.', 'woo-asaas' ), 'https://www.asaas.com/' );

		$this->type = new Credit_Card_Type();
		$this->init_logger();
		$this->admin_settings    = new Credit_Card_Settings( $this );
		$this->validation_errors = new \WP_Error();

		parent::__construct();

		add_action( 'woocommerce_thankyou_' . $this->id, array( $this, 'append_html_to_thankyou_page' ) );
	}

	/**
	 * Add ticker URL to thankyou page.
	 *
	 * @param  int $order_id WC Order id.
	 * @return void
	 */
	public function append_html_to_thankyou_page( $order_id ) {
		$order = new Order( $order_id );

		WC_Asaas::get_instance()->get_template_file( 'order/credit-card-thankyou.php', array( 'order' => $order ) );
	}

	/**
	 * Process a newly credit card order to Asaas API.
	 *
	 * @param  int $order_id WC Order id.
	 * @return array
	 */
	public function process_payment( $order_id ) {
		$order       = new Order( $order_id );
		$wc_order    = $order->get_wc();
		$token       = null;
		$total       = $wc_order->get_total();
		$customer    = $this->get_customer();
		$posted_data = $this->get_posted_data();
		$data        = null;

		// Legacy code support.
		$id        = version_compare( WC()->version, '3.0.0', '<' ) ? $wc_order->id : $wc_order->get_id();
		$remote_ip = version_compare( WC()->version, '3.0.0', '<' ) ? $wc_order->customer_ip_address : $wc_order->get_customer_ip_address();

		// Verify if has selected credit card existent.
		if ( $this->is_one_click_buy() && 'credit-card-new' !== $posted_data['asaas_cc_options'] ) {
			$meta       = $customer->get_meta();
			$card_index = preg_replace( '/[^0-9]+/', '', $posted_data['asaas_cc_options'] );

			if ( empty( $meta['credit_cards'][ $card_index ] ) ) {
				$this->send_checkout_failure(
					array(
						__( 'Are you sure that this card exists?', 'woo-asaas' ),
					)
				);
			}

			$token = $meta['credit_cards'][ $card_index ]['creditCardToken'];
		}

		$payment_data = array(
			'customer'          => $this->asaas_customer_id,
			'billingType'       => 'CREDIT_CARD',
			'value'             => $total,
			'dueDate'           => date( 'Y-m-d' ),
			'externalReference' => $id,
			'remoteIp'          => $remote_ip,
			/* translators: %d: the order id  */
			'description'       => sprintf( __( 'Order %d', 'woo-asaas' ), $id ),
		);

		// If has token, isn't necessary data credit card and holder info.
		if ( $token ) {
			$payment_data['creditCardToken'] = $token;
		} else {
			$payment_data['creditCard'] = array(
				'holderName'  => isset( $posted_data['asaas_cc_name'] ) ? $posted_data['asaas_cc_name'] : '',
				'number'      => isset( $posted_data['asaas_cc_number'] ) ? $posted_data['asaas_cc_number'] : '',
				'expiryMonth' => isset( $posted_data['asaas_cc_expiration_month'] ) ? $posted_data['asaas_cc_expiration_month'] : '',
				'expiryYear'  => isset( $posted_data['asaas_cc_expiration_year'] ) ? $posted_data['asaas_cc_expiration_year'] : '',
				'ccv'         => isset( $posted_data['asaas_cc_security_code'] ) ? $posted_data['asaas_cc_security_code'] : '',
			);

			$payment_data['creditCardHolderInfo'] = Customer::extract_data_from_order( $wc_order );
		}

		$installments = absint( $posted_data['asaas_cc_installments'] );
		if ( 1 < $installments ) {
			$installments_helper = new Installments_Helper();

			$payment_data['installmentCount'] = $installments;
			$payment_data['installmentValue'] = $installments_helper->calculate_installment( $total, $installments );
		}

		/* @var Response $response The API response. */
		$response = apply_filters( 'woocommerce_asaas_process_payment_api_response', $this->api->payments()->create( $payment_data ), $payment_data );

		if ( is_a( $response, Error_Response::class ) ) {
			// print the messages because the payment handle is after the checkout validation.
			$this->send_checkout_failure_response( $response );
		}

		$json = $response->get_json();

		// Add credit card to customer meta data.
		if ( ! $order->is_guest() && ! empty( $json->creditCard ) ) { // phpcs:ignore WordPress.NamingConventions
			$customer->add_credit_card( $json->creditCard ); // phpcs:ignore WordPress.NamingConventions.ValidVariableName
		}

		$order->set_meta_data( $json );

		if ( in_array( $json->status, array( 'RECEIVED', 'CONFIRMED' ), true ) ) {
			$order->complete();
		}

		return array(
			'result'   => 'success',
			'redirect' => $this->get_return_url( $order->get_wc() ),
		);
	}

	/**
	 * Get payment fields
	 *
	 * - Verify if customer has credit card registered to display the one click buy options.
	 * - Verify if installments is allowed and calculate the quantity installments must be showed.
	 *
	 * @return array fields
	 */
	private function get_payment_fields() {
		$customer = $this->get_customer();

		$fields             = array();
		$fields['asaas_cc'] = array(
			'type' => Card::get_instance()->get_type(),
		);

		if ( $this->is_one_click_buy() ) {
			$meta    = $customer->get_meta();
			$options = array();
			foreach ( $meta['credit_cards'] as $i => $card ) {
				$options[ 'credit-card-' . $i ] = WC_Asaas::get_instance()->get_template_file( 'one-click-buy-option.php', array( 'card' => $card ), true );
			}

			$fields['asaas_cc'] = array(
				'type'    => One_Click_Options::get_instance()->get_type(),
				'label'   => __( 'Select a credit card', 'woo-asaas' ),
				'options' => $options,
				'id'      => 'asaas_cc',
			);
		}

		$fields['asaas_cc_installments'] = array(
			'type'     => 'select',
			'label'    => __( 'Installments', 'woo-asaas' ),
			'required' => true,
			'class'    => array( 'asaas_field_cc' ),
			'id'       => 'asaas-cc-installments',
		);

		// Legacy code support.
		$cart  = wc()->cart;
		$total = version_compare( WC()->version, '3.2.0', '<' ) ? $cart->total : $cart->get_total( false );

		$installments_helper   = new Installments_Helper();
		$max_installments      = absint( $this->settings['max_installments'] );
		$min_installment_value = preg_replace( '/,/', '.', $this->settings['min_installment_value'] );
		$min_installment_value = floatval( $min_installment_value );
		$min_installment_value = max( $min_installment_value, 10 );
		$installments          = $installments_helper->get_installment_list( $total, $max_installments, $min_installment_value );

		// Create the installment options.
		foreach ( $installments as $i => $value ) {
			$fields['asaas_cc_installments']['options'][ $i ] = sprintf( '%dx %s', $i, strip_tags( wc_price( $value ) ) );
		}

		return apply_filters(
			'woocommerce_asaas_cc_payment_fields',
			$fields,
			$this
		);
	}

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @see \WC_Payment_Gateway::payment_fields()
	 */
	public function payment_fields() {
		$description = $this->get_description();
		if ( ! empty( $description ) ) {
			echo wp_kses_post( wpautop( wptexturize( $description ) ) );
		}

		$fields = '';
		foreach ( $this->get_payment_fields() as $key => $args ) {
			$args['return'] = true;
			$fields        .= woocommerce_form_field( $key, $args );
		}

		// Transform paragraphs in divs to avoid HTML markup errors.
		// phpcs:ignore WordPress.XSS.EscapeOutput.OutputNotEscaped
		echo preg_replace( array( '/<p/', '/<\/p>/' ), array( '<div', '</div>' ), $fields );
	}

	/**
	 * Get payment form posted data
	 *
	 * @return array The posted data sanitized.
	 */
	public function get_posted_data() {
		$payment_fields = $this->get_payment_fields();
		$data           = array();

		foreach ( $payment_fields as $key => $field ) {
			$field_type = WC_Asaas::get_instance()->get_form_field_object_from_type( $field['type'] );

			if ( is_null( $field_type ) ) {
				$data[ $key ] = isset( $_POST[ $key ] ) ? sanitize_text_field( wp_unslash( $_POST[ $key ] ) ) : ''; // phpcs:ignore WordPress.CSRF.NonceVerification.NoNonceVerification
				continue;
			}

			$field_type->process_data( $key, $field, $data );
		}

		return apply_filters( 'woocommerce_asaas_cc_posted_data', $data );
	}

	/**
	 * Validate checkout fields
	 */
	public function validate_fields() {
		$validation_helper = new Validation_Helper();
		$data              = $this->get_posted_data();

		$this->checkout_card_brand( $data );
		$validation_helper->validate_fields( $this, $this->get_payment_fields(), $data );

		// Legacy code support.
		if ( version_compare( WC()->version, '3.0.0', '<' ) ) {
			foreach ( $this->validation_errors->get_error_messages() as $message ) {
				wc_add_notice( $message, 'error' );
			}

			remove_action( 'woocommerce_after_checkout_validation', array( $this, 'add_checkout_validation_errors' ), 99 );
		}

		return parent::validate_fields();
	}

	/**
	 * Get the checkout card brand from the card number
	 *
	 * @param array $data The submitted data.
	 * @return array The card brand definition.
	 */
	public function checkout_card_brand( $data ) {
		if ( is_null( $this->checkout_card_brand ) ) {
			$card_helper               = new Credit_Card_Helper();
			$this->checkout_card_brand = $card_helper->get_card_brand( $this, $data['asaas_cc_number'] );
		}

		return $this->checkout_card_brand;
	}

	/**
	 * Check if payment is with one click buy
	 *
	 * The setting in admin must be checked as yes and the customer must have at least one card registered.
	 *
	 * @param int $user_id The WP user id.
	 * @return boolean True, if the user is enabled to make one click buy. Otherwise, false.
	 */
	public function is_one_click_buy( $user_id = null ) {
		if ( 'yes' !== $this->settings['one_click_buy'] ) {
			return false;
		}

		$customer = $this->get_customer( $user_id );
		if ( is_null( $customer ) ) {
			return false;
		}

		$meta = $customer->get_meta();
		return ! empty( $meta['credit_cards'] );
	}

	/**
	 * The maximum installments allowed in a order
	 *
	 * @return int The max installments
	 */
	public function get_max_installments() {
		return apply_filters( 'woocommerce_asaas_cc_max_installments', 12 );
	}

	/**
	 * The minimum value for each installment
	 *
	 * @return number The installment value
	 */
	public function get_min_installment_value() {
		return apply_filters( 'woocommerce_asaas_cc_min_installment_value', 10 );
	}

	/**
	 * Validate the max installments settings field to stay in the in a range between 0 and the value of
	 * get_max_installments function
	 *
	 * This function is called by WooCommerce core.
	 *
	 * @param string $key The max installments key.
	 * @param string $value The input value.
	 * @return int The value sanitized.
	 */
	public function validate_max_installments_field( $key, $value ) {
		$value = absint( $value );

		$max_installments = $this->get_max_installments();
		if ( $max_installments < $value ) {
			return $max_installments;
		}

		return $value;
	}

	/**
	 * Validate the minimum intallment value setting to be more than the value returned by get_min_installment_value
	 *
	 * This function is called by WooCommerce core.
	 *
	 * @param string $key The min installments value key.
	 * @param string $value The input value.
	 * @return string The value sanitized.
	 */
	public function validate_min_installment_value_field( $key, $value ) {
		$value = floatval( str_replace( ',', '.', $value ) );

		$min_installment_value = $this->get_min_installment_value();
		if ( $min_installment_value > $value ) {
			return $min_installment_value;
		}

		return str_replace( '.', ',', $value );
	}
}
