<?php
/**
 * Gateway base class
 *
 * @package WooAsaas
 */

namespace WC_Asaas\Gateway;

use WC_Asaas\Api\Api;
use WC_Asaas\WC_Asaas;
use WC_Asaas\Billing_Type\Billing_Type;
use WC_Asaas\Admin\Settings\Settings;
use WC_Asaas\Log\Logger;
use WC_Asaas\Helper\WP_Error_Helper;
use WC_Asaas\Meta_Data\Customer;
use WC_Asaas\Api\Response;
use WC_Asaas\Api\Response\Error_Response;
use WC_Asaas\Log\Logger_Legacy;

/**
 * Gateway base class
 */
abstract class Gateway extends \WC_Payment_Gateway {

	/**
	 * Asaas API wrapper.
	 *
	 * @var Api
	 */
	protected $api;

	/**
	 * The API key
	 *
	 * @var Api
	 */
	protected $api_key;

	/**
	 * Gateway billing type
	 *
	 * @var Billing_Type
	 */
	protected $type;

	/**
	 * Gateway admin settings
	 *
	 * @var Settings
	 */
	protected $admin_settings;

	/**
	 * Gateway logger
	 *
	 * @var Logger
	 */
	protected $logger;

	/**
	 * Payment form validation errors
	 *
	 * @var \WP_Error
	 */
	protected $validation_errors;

	/**
	 * The Asaas customer id used to process the payment
	 *
	 * @var string
	 */
	protected $asaas_customer_id;

	/**
	 * Request customer cache
	 *
	 * @var Customer
	 */
	protected $customer;

	/**
	 * Initialize the form fields, settings and add hook to process the settings form
	 */
	public function __construct() {
		$this->init_form_fields();
		$this->init_settings();
		$this->setup_api();

		$this->title       = $this->get_option( 'title' );
		$this->description = $this->get_option( 'description' );

		$this->validation_errors = new \WP_Error();

		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
		add_action( 'woocommerce_settings_checkout', array( $this, 'replace_config_url' ) );

		$setting_option = "woocommerce_{$this->id}_settings";
		add_action( "update_option_{$setting_option}", array( $this, 'enable_disable_customer_notifications' ), 10, 3 );

		add_action( 'woocommerce_checkout_update_user_meta', array( $this, 'set_customer' ), 10, 2 );
		add_action( 'woocommerce_after_checkout_validation', array( $this, 'add_checkout_validation_errors' ), 99, 2 );
	}

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @see \WC_Settings_API::init_form_fields()
	 */
	public function init_form_fields() {
		$this->form_fields = $this->admin_settings->fields();
	}

	/**
	 * Setup API wrapper.
	 *
	 * @return void
	 */
	private function setup_api() {
		$this->api_key = $this->get_option( 'api_key' );
		$this->api     = new Api( $this );
	}

	/**
	 * Get API key setting
	 *
	 * @return string The API key.
	 */
	public function get_api_key() {
		return $this->api_key;
	}

	/**
	 * Get Asaas customer metadata object
	 *
	 * @param int $user_id (Optional) The user ID. Default: Current logged user.
	 * @return Customer|null The customer object. Null, if the $user_id doesn't exist or user not logged in.
	 */
	public function get_customer( $user_id = null ) {
		if ( ! is_null( $this->customer ) ) {
			return $this->customer;
		}

		$user_id = is_null( $user_id ) ? get_current_user_id() : $user_id;

		if ( 0 === $user_id ) {
			return null;
		}

		$this->customer = new Customer( $user_id );
		return $this->customer;
	}

	/**
	 * Get the gateway billing type
	 *
	 * @return Billing_Type The gateway billing type.
	 */
	public function get_type() {
		return $this->type;
	}

	/**
	 * Get gateway admin settings
	 *
	 * @return Settings The gateway admin settings.
	 */
	public function get_admin_settings() {
		return $this->admin_settings;
	}

	/**
	 * Get gateway logger
	 *
	 * @return Logger The gateway logger.
	 */
	public function get_logger() {
		return $this->logger;
	}

	/**
	 * Initialize the logger
	 */
	public function init_logger() {
		// Legacy code support.
		if ( version_compare( WC()->version, '3.0.0', '<' ) ) {
			$this->logger = new Logger_Legacy( $this );
			return;
		}

		$this->logger = new Logger( $this );
	}

	/**
	 * The error code for checkout validation errors
	 *
	 * @return string
	 */
	public function get_error_code() {
		return 'asaas-invalid';
	}

	/**
	 * Process shared options between the plugin gateways
	 *
	 * {@inheritDoc}
	 *
	 * @see \WC_Settings_API::process_admin_options()
	 */
	public function process_admin_options() {
		parent::process_admin_options();

		$shared_fields = $this->get_shared_fields();
		if ( 0 === count( $shared_fields ) ) {
			return;
		}

		$gateways = WC_Asaas::get_instance()->get_gateways();
		foreach ( $gateways as $id => $gateway ) {
			if ( $this->id === $gateway->id ) {
				continue;
			}

			$this->process_admin_shared_options( $gateway, $shared_fields );
		}
	}

	/**
	 * Get fields that are shared between the plugin gateways
	 *
	 * @return array The shared fields
	 */
	public function get_shared_fields() {
		$shared_fields = array();

		foreach ( $this->get_form_fields() as $field ) {
			if ( isset( $field['shared'] ) && true === $field['shared'] ) {
				$shared_fields[ $field['id'] ] = $field;
			}
		}

		return $shared_fields;
	}

	/**
	 * Process shared options in another gateway
	 *
	 * This function is very similar WC_Settings_API::process_admin_options function. The difference that is processed just the shared fields.
	 *
	 * @param Gateway $other_gateway The another gateway.
	 * @param array   $shared_fields The current gateway shared fields list.
	 * @return bool True if settings was updated. False, otherwise.
	 */
	public function process_admin_shared_options( $other_gateway, $shared_fields ) {
		$post_data = $this->get_post_data();

		foreach ( $shared_fields as $key => $field ) {
			if ( 'title' !== $this->get_field_type( $field ) ) {
				try {
					$other_gateway->settings[ $key ] = $this->get_field_value( $key, $field, $post_data );
				} catch ( \Exception $e ) {
					$this->add_error( $e->getMessage() );
				}
			}
		}

		return update_option( $other_gateway->get_option_key(), apply_filters( 'woocommerce_settings_api_sanitized_fields_' . $other_gateway->id, $other_gateway->settings ) );
	}

	/**
	 * Replace config URL after update the settings
	 */
	public function replace_config_url() {
		$this->form_fields = $this->admin_settings->replace_config_url();
	}

	/**
	 * Integrate the order with Asaas
	 *
	 * Create or update an Asaas customer to process the order. If the API returns some error, the process will be
	 * stopped and the erros will be returned to show in the checkout page.
	 *
	 * @param int   $customer_id The WooCommerce customer id.
	 * @param array $data The checkout data.
	 */
	public function set_customer( $customer_id, $data ) {
		if ( $this->id !== $data['payment_method'] ) {
			return;
		}

		$customer                              = null;
		$action                                = 'create';
		$params                                = array();
		$customer_data                         = Customer::extract_data_from_checkout( $data );
		$customer_data['notificationDisabled'] = 'yes' !== $this->settings['notification'];

		if ( 0 < $customer_id ) {
			$customer                           = $this->get_customer( $customer_id );
			$customer_data['externalReference'] = $customer_id;

			if ( $customer->has_meta() ) {
				$action   = 'update';
				$meta     = $customer->get_meta();
				$params[] = $meta['id'];
			}
		}

		$params[] = $customer_data;

		$params = apply_filters( 'woocommerce_asaas_set_customer_params', $params, $customer, $action );
		$action = apply_filters( 'woocommerce_asaas_set_customer_action', $action, $customer, $params );

		/* @var Response $response The API response. */
		$response = apply_filters( 'woocommerce_asaas_set_customer_api_response', call_user_func_array( array( $this->api->customers(), $action ), $params ) );

		if ( is_a( $response, Error_Response::class ) ) {
			// print the messages because the customer handle is after the checkout validation.
			$this->send_checkout_failure_response( $response );
		}

		$this->asaas_customer_id = $response->id;

		if ( ! is_null( $customer ) ) {
			$customer->set_meta( 'id', $response->id );
		}
	}

	/**
	 * Add error to be show together another checkout errors
	 *
	 * @param \WP_Error $error The checkout error.
	 */
	public function add_validation_errors( $error ) {
		$wp_error_helper = new WP_Error_Helper();
		$wp_error_helper->merge( $this->validation_errors, $error );
	}

	/**
	 * Add the gateway checkout errors to WooCommerce error area.
	 *
	 * This function is called by `woocommerce_after_checkout_validation` hook.
	 *
	 * Create the error object if the $erros parameter is false.
	 *
	 * @param array             $data The submitted data.
	 * @param \WP_Error|boolean $errors The checkout errors. Default: false.
	 */
	public function add_checkout_validation_errors( $data, $errors = false ) {
		if ( ! $errors ) {
			$errors = new \WP_Error();
		}

		$wp_error_helper = new WP_Error_Helper();
		$wp_error_helper->merge( $errors, $this->validation_errors );
	}

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @see \WC_Payment_Gateway::validate_fields()
	 */
	public function validate_fields() {
		return empty( $this->validation_errors->get_error_codes() );
	}

	/**
	 * Send API warnings to the WooCommerce checkout notice area
	 *
	 * @param Response $response The API response.
	 */
	public function send_checkout_failure_response( $response ) {
		$messages = array();
		foreach ( $response->get_errors()->get_error_messages() as $message ) {
			$messages[] = $message;
		}

		$this->send_checkout_failure( $messages );
	}

	/**
	 * Stop the checkout processing and send errors to checkout page
	 *
	 * Based on WC_Checkout::send_ajax_failure_response(). Just work with AJAX requests.
	 *
	 * @see \WC_Checkout::send_ajax_failure_response()
	 *
	 * @param array $messages The messages to be showed in the checkout.
	 */
	public function send_checkout_failure( $messages ) {
		if ( ! is_ajax() ) {
			return;
		}

		foreach ( $messages as $message ) {
			wc_add_notice( $message, 'error' );
		}

		ob_start();
		wc_print_notices();
		$output = ob_get_clean();

		$response = array(
			'result'   => 'failure',
			'messages' => $output,
		);

		wp_send_json( $response );
	}

	/**
	 *
	 * {@inheritDoc}
	 *
	 * @see \WC_Payment_Gateway::is_available()
	 */
	public function is_available() {
		if ( ! parent::is_available() ) {
			return false;
		}

		if ( empty( $this->settings['min_total'] ) ) {
			return true;
		}

		$total     = wc()->cart->get_total( false );
		$min_total = floatval( str_replace( ',', '.', $this->settings['min_total'] ) );

		return $total > $min_total;
	}

	/**
	 * Update all customers notificationDisabled, if the value is changed in the settings form
	 *
	 * Not process if it isn`t the gateway that the options is updated to avoid process the same request many times.
	 * These would happen because notification is a shared option.
	 *
	 * @param string $old_value The value before the change.
	 * @param string $value The new value.
	 * @param string $option The option name that was updated.
	 */
	public function enable_disable_customer_notifications( $old_value, $value, $option ) {
		$post_data = $this->get_post_data();
		if ( ! isset( $post_data[ "woocommerce_{$this->id}_enabled" ] ) ) {
			return;
		}

		if ( "woocommerce_{$this->id}_settings" === $option && $old_value['notification'] !== $value['notification'] ) {
			$notification_disabled = 'yes' !== $value['notification'];
			$customers             = Customer::get_customers();

			foreach ( $customers as $customer ) {
				$this->api->customers()->update(
					$customer->id, array(
						'notificationDisabled' => $notification_disabled,
					)
				);
			}
		}
	}

	/**
	 * Validate the minimum total accepted
	 *
	 * The total must be positive.
	 *
	 * @param string $key The min installments value key.
	 * @param string $value The input value.
	 * @return string The value sanitized.
	 */
	public function validate_min_total_field( $key, $value ) {
		$value = floatval( str_replace( ',', '.', $value ) );

		if ( 0 > $value ) {
			return 0;
		}

		return str_replace( '.', ',', $value );
	}
}
