<?php
/**
 * Ticket class
 *
 * @package WooAsaas
 */

namespace WC_Asaas\Gateway;

use WC_Asaas\Meta_Data\Order;
use WC_Asaas\Admin\Settings\Ticket as Ticket_Settings;
use WC_Asaas\Billing_Type\Ticket as Ticket_Type;
use WC_Asaas\WC_Asaas;

/**
 * Asaas ticket gateway
 */
class Ticket extends Gateway {
	/**
	 * Init the gateway
	 */
	public function __construct() {
		$this->id           = 'asaas-ticket';
		$this->has_fields   = false;
		$this->method_title = __( 'Asaas Ticket', 'woo-asaas' );
		/* translators: %s: Asaas website URL  */
		$this->method_description = sprintf( __( 'Use <a href="%s">Asaas</a> to allow your customer buy your products by ticket.', 'woo-asaas' ), 'https://www.asaas.com/' );

		$this->type = new Ticket_Type();
		$this->init_logger();
		$this->admin_settings = new Ticket_Settings( $this );

		parent::__construct();

		add_action( 'woocommerce_thankyou_' . $this->id, array( $this, 'append_html_to_thankyou_page' ) );
	}

	/**
	 * Add ticker URL to thankyou page.
	 *
	 * @param  int $order_id WC Order id.
	 * @return void
	 */
	public function append_html_to_thankyou_page( $order_id ) {
		$order = new Order( $order_id );

		WC_Asaas::get_instance()->get_template_file( 'order/ticket-thankyou.php', array( 'order' => $order ) );
	}

	/**
	 * Create due date for ticket
	 *
	 * Check in Asaas the minimum due date for the ticket. If the validity date is less than the minimum, use the
	 * minimum as due date.
	 *
	 * @return \DateTime The due date.
	 */
	private function create_due_date() {
		$today         = date( 'Y-m-d' );
		$validity_days = apply_filters( 'woocommerce_asaas_default_ticket_validity_days', 3 );
		if ( isset( $this->settings['validity_days'] ) ) {
			$validity_days = intval( $this->settings['validity_days'] );
		}

		$due_date = new \DateTime( sprintf( '+ %d days', $validity_days ) );

		$mininum_due_date = $this->api->payments()->minimum_bank_slip_due_date();
		$mininum_due_date = $mininum_due_date->get_json()->minimumDueDate;
		$mininum_due_date = new \DateTime( $mininum_due_date );

		return $mininum_due_date > $due_date ? $mininum_due_date : $due_date;
	}

	/**
	 * Process a ticket order to Asaas API
	 *
	 * If is a new payment processing of the an existing order and the ticket is the last try, is unnecessary generate
	 * a new payment. The same ticket is used.
	 *
	 * @param  int $order_id WC Order id.
	 * @return array
	 */
	public function process_payment( $order_id ) {
		$order    = new Order( $order_id );
		$wc_order = $order->get_wc();

		$meta = $order->get_meta_data();
		if ( $meta ) {
			if ( $this->type->get_id() === $meta->billingType ) { // phpcs:ignore WordPress.NamingConventions.ValidVariableName.NotSnakeCaseMemberVar
				return $this->success( $wc_order );
			}

			$this->asaas_customer_id = $meta->customer;
		}

		$total = $wc_order->get_total();

		// Legacy code support.
		$id = version_compare( WC()->version, '3.0.0', '<' ) ? $wc_order->id : $wc_order->get_id();

		$payment = $this->api->payments()->create(
			array(
				'customer'          => $this->asaas_customer_id,
				'billingType'       => $this->type->get_id(),
				'value'             => $total,
				'dueDate'           => $this->create_due_date()->format( 'Y-m-d' ),
				'externalReference' => $id,
				/* translators: %d: the order id  */
				'description'       => sprintf( __( 'Order %d', 'woocommerce-asaas' ), $id ),
			)
		);

		$order->set_meta_data( $payment->get_json() );

		return $this->success( $wc_order );
	}

	/**
	 * Return a WooCommerce success response redirecting to order page
	 *
	 * @param \WC_Order $wc_order The WooCommerce order object.
	 * @return string[] The response data.
	 */
	private function success( $wc_order ) {
		return array(
			'result'   => 'success',
			'redirect' => $this->get_return_url( $wc_order ),
		);
	}
}
