<?php
/**
 * File for class Webhook
 *
 * @package WooAsaas
 */

namespace WC_Asaas\Webhook;

use WC_Asaas\WC_Asaas;
use WC_Asaas\Gateway\Gateway;

/**
 * Webhook
 */
class Webhook {

	/**
	 * Attributes of the webhook
	 *
	 * @var object $data
	 */
	private $data;

	/**
	 * Order related to webhook
	 *
	 * @var \WC_Order
	 */
	private $order;

	/**
	 * Plugin main class
	 *
	 * @var WC_Asaas
	 */
	private $wc_asaas;

	/**
	 * Value with discounts and fee
	 *
	 * @var float
	 */
	private $value_total;

	/**
	 * Discounts values
	 *
	 * @var float
	 */
	private $discounts_values;

	const PREFIX_LOG = 'Asaas: ';

	const PAYMENT_CREATED = 'PAYMENT_CREATED';

	const PAYMENT_UPDATED = 'PAYMENT_UPDATED';

	const PAYMENT_CONFIRMED = 'PAYMENT_CONFIRMED';

	const PAYMENT_RECEIVED = 'PAYMENT_RECEIVED';

	const PAYMENT_OVERDUE = 'PAYMENT_OVERDUE';

	const PAYMENT_REFUNDED = 'PAYMENT_REFUNDED';

	const PAYMENT_DELETED = 'PAYMENT_DELETED';

	const PAYMENT_RESTORED = 'PAYMENT_RESTORED';

	/**
	 * Initialize the object
	 *
	 * @param Gateway   $gateway The payment gateway.
	 * @param \WC_Order $order The webhook that will be processed.
	 * @param \stdClass $data The webhook data.
	 */
	public function __construct( Gateway $gateway, \WC_Order $order, \stdClass $data ) {
		$this->gateway = $gateway;
		$this->order   = $order;
		$this->data    = $data;

		$this->order->set_date_modified( new \WC_DateTime( date( 'Y-m-d H:i:s' ) ) );

		add_action( 'woocommerce_order_after_calculate_totals', array( $this, 'save_values' ) );
	}

	/**
	 * Magic function to access attributes
	 *
	 * @param  string $name name of the attribute.
	 * @return mixed The value of the attribute.
	 */
	public function __get( $name ) {
		return $this->data->{$name};
	}

	/**
	 * Process the event according to its type
	 *
	 * @throws \Exception Case event not found.
	 */
	public function process_event() {
		switch ( $this->event ) {
			case Webhook::PAYMENT_CONFIRMED:
				$this->on_payment_confirmed();
				break;

			case Webhook::PAYMENT_CREATED:
				$this->on_payment_created();
				break;

			case Webhook::PAYMENT_DELETED:
				$this->on_payment_deleted();
				break;

			case Webhook::PAYMENT_OVERDUE:
				$this->on_payment_overdue();
				break;

			case Webhook::PAYMENT_RECEIVED:
				$this->on_payment_received();
				break;

			case Webhook::PAYMENT_REFUNDED:
				$this->on_payment_refunded();
				break;

			case Webhook::PAYMENT_RESTORED:
				$this->on_payment_restored();
				break;

			case Webhook::PAYMENT_UPDATED:
				$this->on_payment_updated();
				break;

			default:
				/* translators: %s: event name  */
				die( esc_html( sprintf( __( 'Untreated event: %s', 'woo-asaas' ), $this->event ) ) );
		}
	}

	/**
	 * Method used when payment is confirmed
	 */
	private function on_payment_confirmed() {
		$this->order->update_status( 'completed' );
		$this->add_order_note( 'Paymen confirmed.' );
	}

	/**
	 * Method used when payment is created
	 */
	private function on_payment_created() {
		$this->order->update_status( 'pending-payment' );
		$this->add_order_note( 'Payment created.' );
	}

	/**
	 * Method used when payment is deleted
	 */
	private function on_payment_deleted() {
		$this->order->update_status( 'cancelled' );
		$this->add_order_note( 'Payment deleted.' );
	}

	/**
	 * Method used when payment is overdue
	 */
	private function on_payment_overdue() {
		$this->order->update_status( 'falied' );
		$this->add_order_note( 'Payment overdue.' );
	}

	/**
	 * Method used when payment is received
	 */
	private function on_payment_received() {
		$order_data = $this->order->get_data();

		if ( $order_data['total'] !== $this->payment->value ) {
			$this->order->set_total( $this->payment->value );
		}

		// Value for change total of order.
		$this->value_total = $this->payment->value;

		// When payment is overdue.
		if ( date( 'Y-m-d' ) > $this->payment->dueDate ) {

			// Add tax for fine.
			if ( $this->payment->fine && $this->payment->fine->value ) {

				// Calculate value.
				$value = ( $this->payment->value * ( $this->payment->fine->value / 100 ) );

				// Add fee item in order.
				$fine_tax = new \WC_Order_Item_Fee();
				$fine_tax->set_total( $value );
				$fine_tax->set_name( 'Fine tax' );
				$this->order->add_item( $fine_tax );

				// Sum the value of the total fee.
				$this->value_total += $value;
			}

			if ( $this->payment->interest && $this->payment->interest->value ) {

				// Add fee item in order.
				$interest = new \WC_Order_Item_Fee();
				$interest->set_name( 'Interest' );

				if ( 'PERCENTAGE' === $this->payment->interest->type ) {
					// Calculate value.
					$value = ( $this->payment->value * ( $this->payment->interest->value / 100 ) );

					$interest->set_total( $value );

					// Sum the fee in total.
					$this->value_total += $value;
				} else {
					$interest->set_totat( $this->payment->interest->value );

					// Sum the fee in total.
					$this->value_total += $this->payment->fine->value;
				}

				$this->order->add_item( $interest );
			}
		} elseif ( $this->payment->discount && $this->payment->discount->value ) {

			// Add discount in order.
			if ( 'PERCENTAGE' === $this->payment->discount->type ) {

				// Calculate value.
				$value = ( $this->payment->value * ( $this->payment->discount->value / 100 ) );

				$this->discounts_values = $value;

				// Sum the discount in total.
				$this->value_total -= $value;

				$this->order->set_discount_total( $value );

			} else {
				$this->discounts_values = $value;

				$this->order->set_discount_total( $this->payment->discount->value );

				// Sum the discount in total.
				$this->value_total -= $this->payment->discount->value;
			}
		}

		// Calculate totals ordem.
		$this->order->calculate_totals();

		$this->order->update_status( 'processing' );
		$this->add_order_note( 'Payment received.' );
	}

	/**
	 * Method used when payment is refunded
	 */
	private function on_payment_refunded() {
		$this->order->update_status( 'refunded' );
		$this->add_order_note( 'Payment refunded.' );
	}

	/**
	 * Method used when payment is restored
	 */
	private function on_payment_restored() {
		$this->order->update_status( 'pending-payment' );
		$this->add_order_note( 'Payment restored.' );
	}

	/**
	 * Method used when payment is updated
	 */
	private function on_payment_updated() {
		$this->order->set_total( $this->payment->value );
		$this->order->update_status( 'pending-payment' );
		$this->add_order_note( 'Payment updated' );
	}

	/**
	 * Save totals values
	 */
	public function save_values() {
		// Save discount amount after calculation.
		$this->order->set_discount_total( $this->discounts_values );
		// Save total order after calculation.
		$this->order->set_total( $this->value_total );
	}

	/**
	 * Add note
	 *
	 * @param String $note note description.
	 */
	private function add_order_note( $note ) {
		$this->order->add_order_note( self::PREFIX_LOG . ' ' . $note );
	}
}
